#ifndef _TIMING_HPP_
#define _TIMING_HPP_
#include <cstdint>

typedef union {
	uint64_t int64;
	struct {
		uint32_t lo;
		uint32_t hi;
	} int32;
} tsc_counter;

#define START_TSC(cpu_c) \
	asm volatile ( \
			"cpuid\n\t" \
			"rdtsc\n\t" \
			"mov %%edx, %0\n\t" \
			"mov %%eax, %1\n\t" \
			: "=r" ((cpu_c).int32.hi), "=r"((cpu_c).int32.lo) \
			: /* no input */ \
			: "%rax", "%rbx", "%rcx", "%rdx" \
	)
#define STOP_TSC(cpu_c) \
	asm volatile ( \
			"rdtscp\n\t" \
			"mov %%edx, %0\n\t" \
			"mov %%eax, %1\n\t" \
			"cpuid\n\t" \
			: "=r" ((cpu_c).int32.hi), "=r"((cpu_c).int32.lo) \
			: /* no input */ \
			: "%rax", "%rbx", "%rcx", "%rdx" \
	)

uint64_t start_tsc() {
	tsc_counter start;
	START_TSC(start);
	return start.int64;
}

uint64_t stop_tsc(uint64_t start) {
	tsc_counter end;
	STOP_TSC(end);
	return end.int64 - start;
}

#endif //_TIMING_HPP_
