// paste -d ' ' <(cat tmpi_3procs.out | tr -s ' ' |  cut  -d ' ' -f 2,4) <(cat mpi_3procs.out
// | tr -s ' ' | cut -d ' ' -f 3) > out_3procs

#define EXT_USESCIBENCH

#define MEASALL

#include <cmath>
#include <chrono>

#include <sstream>
#include <string>
#include <iomanip>
#include <liblsb.h>
#include <tmpi.h>
#include <debug/debugger.h>
#include <cassert>
#include <iostream>

//    _________________________ ... __________________________
//   |          :          :           :          :          |
//   |          :          :           :          :          |
//   |          :          :           :          :          |  pygnum-1
//   |          :          :           :          :          |
//   |----------:----------:--- ... ---:----------:----------|
//   |          :          :           :          :          |
//   |          :          :           :          :          |  pygnum-2
//   |          :          :           :          :          |
//   |----------:----------:--- ... ---:----------:----------|
//   |          :          :           :          :          |
//   :          '          '           '          '          '
//
//   .          .          .           .          .          .
//   |          :          :           :          :          |
//   |          :          :           :          :          |
//   |----------:----------:--- ... ---:----------:----------|
//   |          :          :           :          :          |
//   |          :          :           :          :          |  1
//   |          :          :           :          :          |  
//   |----------:----------:--- ... ---:----------:----------|  | pygnum
//   | 3  4  5  : 9 10 11  :           :          :          |
//   |          :          :           :          :          |  0
//   | 0  1  2  : 6  7  8  :           :          :          |
//   |__________:__________:___ ... ___:__________:__________|
//
//   --> pxgnum
//         0          1                  pxgnum-2   pxgnum-1

thread_local double wtime_task2task_isend = 0;

// rank is the global rank of this task. Global position pxg/pyg from 0..n,
// local position pxl/pyl from 0..n, Global size pxgnum/pygnum from 0..n,
// local size pxlnum/pylnum, task size pxsz, pysz and msg arrays msg/msg_recv. N is the border size
// nsperkele = "number of clock cycles per grid cell"*1024 (because it is divided by 1024 in the code)
inline void benchmark(int rank, int pxg, int pyg, int pxl, int pyl, int pxgnum, int pygnum,
		int pxlnum, int pylnum, int pxsz, int pysz, int N, double* msg, double* msg_recv,
		int nsperkele, int comp_enabled, int exchange_enabled, bool timing) {

	// we need this to be volatile in order to exclude any cache effects and other optimizations.
	// the loop with res must need the same amount of time for EACH configuration
	volatile int res = msg_recv[0];
	if (timing)
		LSB_Res();

	const int mx = pxsz * pysz;

	auto rdtsc = []() -> uint64_t {
		uint64_t hi, lo;
		__asm__ volatile ("rdtsc" : "=a"(lo), "=d"(hi));
		return lo | (hi << 32);
	};
	uint64_t start = rdtsc();
	uint64_t end = start + (uint64_t) mx * nsperkele / 1024;
	int i = 0;
	if (comp_enabled == 1) {
		while (rdtsc() < end) {
			++i;
		}
	}
	msg[0] = i;

	// periodic boundary conditions
	// get left and right tasks (left-most = 0)
	const int left = (pxl > 0 ? rank - 1
			: (pxg > 0 ? rank - pxlnum*(pylnum-1) - 1
			: rank + pxlnum*pylnum*(pxgnum-1) + pxlnum - 1));
	const int right = (pxl < pxlnum - 1 ? rank + 1
			: (pxg < pxgnum - 1 ? rank + pxlnum*(pylnum-1) + 1
			: rank - pxlnum*pylnum*(pxgnum-1) - pxlnum + 1));

	// get above and below tasks (left-most = 0)
	const int below = (pyl > 0 ? rank - pxlnum
			: (pyg > 0 ? rank - pxlnum*pylnum*(pxgnum-1) - pxlnum
			: rank + pxlnum*pylnum*pxgnum*(pygnum-1) + pxlnum*(pylnum-1)));

	const int above = (pyl < pylnum - 1 ? rank + pxlnum
			: (pyg < pygnum - 1 ? rank + pxlnum*pylnum*(pxgnum-1) + pxlnum
			: rank - pxlnum*pylnum*pxgnum*(pygnum-1) - pxlnum*(pylnum-1)));

	if (timing)
		LSB_Rec(1);


///
#ifdef HAHA
//TMPI_Request req1[1] = {};
//if (rank == 0 || rank == 1) {
//TMPI_Irecv(&msg_recv[2*N*pxsz + N*pysz], N*pysz, TMPI_DOUBLE, right, 0, TMPI_COMM_WORLD, req1);
//}

//if (rank == 2 || rank == 3) {
//TMPI_Isend(&msg     [0       ], N*pysz, TMPI_DOUBLE, left,  0, TMPI_COMM_WORLD, req1);
//}
//TMPI_Waitall(1, req1, TMPI_STATUSES_IGNORE);

//return;
#endif
///


	TMPI_Request req[4] = {};
//std::cout << "Before sends\n";std::cout.flush();


// TODO The offsets in the code below are not exactly correct. Need MPI dertype support to make it correct.
// TODO original req offsets were: 2 3 6 7 0 1 4 5
if (exchange_enabled == 1) {
	if (pxl > 0) {
		TMPI_Isend(&msg     [0       ], N*pysz, TMPI_DOUBLE, left,  0, TMPI_COMM_WORLD, req);
// 		if (timing)
// 			LSB_Rec(250);
	} else {
		TMPI_Isend(&msg     [0       ], N*pysz, TMPI_DOUBLE, left,  0, TMPI_COMM_WORLD, req);
// 		if (timing)
// 			LSB_Rec(251);
	}
//std::cout << "Done first send\n";std::cout.flush();

	if (pxl < pxlnum - 1) {
		TMPI_Isend(&msg     [N*pysz  ], N*pysz, TMPI_DOUBLE, right, 0, TMPI_COMM_WORLD, req+1);
// 		if (timing)
// 			LSB_Rec(240);
	} else {
		TMPI_Isend(&msg     [N*pysz  ], N*pysz, TMPI_DOUBLE, right, 0, TMPI_COMM_WORLD, req+1);
// 		if (timing)
// 			LSB_Rec(241);
	}
//std::cout << "In the middle\n";std::cout.flush();
double t2t_start = LSB_HRtime();
	if (pyl < pylnum - 1) {
		//TMPI_Isend(&msg     [2*N*pysz], N*pxsz, TMPI_DOUBLE, above, 0, TMPI_COMM_WORLD, req+6);
// 		if (timing)
// 			LSB_Rec(220);
	} else {
		//TMPI_Isend(&msg     [2*N*pysz], N*pxsz, TMPI_DOUBLE, above, 0, TMPI_COMM_WORLD, req+6);
// 		if (timing)
// 			LSB_Rec(221);
	}
//std::cout << "Done 3rd send\n";std::cout.flush();

	if (pyl > 0) {
		//TMPI_Isend(&msg     [2*N*pysz + N*pxsz], N*pxsz, TMPI_DOUBLE, below, 0, TMPI_COMM_WORLD, req+7);
// 		if (timing)
// 			LSB_Rec(230);
	} else {
		//TMPI_Isend(&msg     [2*N*pysz + N*pxsz], N*pxsz, TMPI_DOUBLE, below, 0, TMPI_COMM_WORLD, req+7);
// 		if (timing)
// 			LSB_Rec(231);
	}
double t2t_end = LSB_HRtime();
wtime_task2task_isend += t2t_end - t2t_start;
//std::cout << "Between\n";std::cout.flush();

	if (pyl < pylnum - 1) {
		//TMPI_Irecv(&msg_recv[0       ], N*pxsz, TMPI_DOUBLE, above, 0, TMPI_COMM_WORLD, req+0);
// 		if (timing)
// 			LSB_Rec(140);
	} else {
		//TMPI_Irecv(&msg_recv[0       ], N*pxsz, TMPI_DOUBLE, above, 0, TMPI_COMM_WORLD, req+0);
// 		if (timing)
// 			LSB_Rec(141);
	}
	if (pyl > 0) {
		//TMPI_Irecv(&msg_recv[N*pxsz  ], N*pxsz, TMPI_DOUBLE, below, 0, TMPI_COMM_WORLD, req+1);
// 		if (timing)
// 			LSB_Rec(150);
	} else {
		//TMPI_Irecv(&msg_recv[N*pxsz  ], N*pxsz, TMPI_DOUBLE, below, 0, TMPI_COMM_WORLD, req+1);
// 		if (timing)
// 			LSB_Rec(151);
	}

	if (pxl > 0) {
		TMPI_Irecv(&msg_recv[2*N*pxsz], N*pysz, TMPI_DOUBLE, left,  0, TMPI_COMM_WORLD, req+2);
// 		if (timing)
// 			LSB_Rec(120);
	} else {
		TMPI_Irecv(&msg_recv[2*N*pxsz], N*pysz, TMPI_DOUBLE, left,  0, TMPI_COMM_WORLD, req+2);
// 		if (timing)
// 			LSB_Rec(121);
	}
	if (pxl < pxlnum - 1) {
		TMPI_Irecv(&msg_recv[2*N*pxsz + N*pysz], N*pysz, TMPI_DOUBLE, right, 0, TMPI_COMM_WORLD, req+3);
// 		if (timing)
// 			LSB_Rec(130);
	} else {
		TMPI_Irecv(&msg_recv[2*N*pxsz + N*pysz], N*pysz, TMPI_DOUBLE, right, 0, TMPI_COMM_WORLD, req+3);
// 		if (timing)
// 			LSB_Rec(131);
	}


	if (timing)
		LSB_Rec(2);
//std::cout << "Before Waitall " << rank << std::endl;std::cout.flush();
	TMPI_Waitall(4, req, TMPI_STATUSES_IGNORE);
	if (timing)
		LSB_Rec(3);
//std::cout << "Returning\n";std::cout.flush();
} // end if (exchange_enabled)

}

#include "include/cache_flush.h"

constexpr double freq = 2.7; //GHz
static thread_local int numpassed = 0;
thread_local double wtime_running = 0, wtime_start, wtime_end;

int main(int argc, char** argv) {
	TMPI_Init(&argc, &argv);
LSB_Rec_disable(); // currently using only the direct timings with LSB_HRtime()
	LSB_Set_Rparam_int("enabled",0 );
	LSB_Set_Rparam_int("type",0 );

	CacheFlush cf(CACHESIZE);

	int rank, size, res;
	(void) res;
	res = TMPI_Comm_rank(TMPI_COMM_WORLD, &rank);
	res = TMPI_Comm_size(TMPI_COMM_WORLD, &size);
std::cout << "My rank is " << rank << " commsize is " << size << std::endl;std::cout.flush();

	int pxlnum = atoi(argv[1]); // x-extent of the local-to-node task-grid
	int pylnum = atoi(argv[2]); // y-extent ...
	int pxgnum = atoi(argv[3]); // x-extent of the node-grid
	int pygnum = atoi(argv[4]); // y-extent ...
	int pxsz = atoi(argv[5]);   // size of the chunk local to this rank
	int pysz = atoi(argv[6]);
	int N = atoi(argv[7]);      // halo depth
	int kmax = atoi(argv[8]);   // cycles*1024 per grid cell
	int runid = atoi(argv[9]);  // simplifies the distinguishing of different runs under the same batch allocation
	int comp_enabled = atoi(argv[10]); // 1=computation enabled, 0=disabled
	int exchange_enabled = atoi(argv[11]);	// 1=halo exchange enabled, 0=disabled

	int rankl = rank % (pxlnum*pylnum); // rank of the task (TMPI-rank) within the node
	int rankg = (rank - rankl) / (pxlnum*pylnum); // rank of the node within the node-grid

	int pxg = rankg % pxgnum; // x-coord of the node in the node-grid
	int pyg = (rankg - pxg) / pxgnum; // y-coord ...
	int pxl = rankl % pxlnum; // x-coord of the task in the local-to-node task-grid
	int pyl = (rankl - pxl) / pxlnum; // y-coord ...
	// The ranks grow in the direction of the X-axis:
	//   |----------|
	//   | 3  4  5  | 
	//   |          |      
	//   | 0  1  2  | 
	//   |__________|
	//   This is true for both the node-grid and the local-to-node task-grid.
	//   Overall mapping is correctly depicted on the diagram at the start of the file.

	assert(pxlnum * pylnum * pxgnum * pygnum == size);
	assert(pxg >= 0 && pxg < pxgnum);
	assert(pyg >= 0 && pyg < pygnum);
	assert(pxl >= 0 && pxl < pxlnum);
	assert(pyl >= 0 && pyl < pylnum);

	double* message      = (double*) memalign(64, (pxsz+2*N)*(pysz+2*N)*sizeof(double));
	double* message_recv = (double*) memalign(64, (pxsz+2*N)*(pysz+2*N)*sizeof(double));

	int task_id = tinympi::tasks::LocalTask::get_current_task()->get_identifier().local.task_id;
	TMPI_Barrier(TMPI_COMM_WORLD);

//std::cout << "Before warmup\n";std::cout.flush();

	cf.flush(); // clear cache
	// warmup
	for (int i = 0; i < 40; ++i) {
		benchmark(rank, pxg, pyg, pxl, pyl, pxgnum, pygnum, pxlnum, pylnum, pxsz, pysz, N, message,
				message_recv, kmax, 1, 1, false);
	}
//std::cout << "After warmup\n";std::cout.flush();
	
	TMPI_Barrier(TMPI_COMM_WORLD);

	for (int i = 0; i <  20; ++i) {
		if (numpassed++ == 0) {
			LSB_Rec(-1);
			LSB_Set_Rparam_int("enabled", 1);
			wtime_start = LSB_HRtime();
		}

		for (int j = 0; j < 100; ++j) {		
			benchmark(rank, pxg, pyg, pxl, pyl, pxgnum, pygnum, pxlnum, pylnum, pxsz, pysz, N,
					message, message_recv, kmax, comp_enabled, exchange_enabled, false); 	 
		}

		if (--numpassed == 0) {
			LSB_Rec(999);
			LSB_Set_Rparam_int("enabled", 0);
			wtime_end = LSB_HRtime(); 
			wtime_running += (wtime_end-wtime_start);
		}
		TMPI_Barrier(TMPI_COMM_WORLD);
	}
	TMPI_Barrier(TMPI_COMM_WORLD);

	if (numpassed++ == 0) {
		std::cout << "Benchmark is currently restricted to node-to-node communication only. Task-to-task is not done.\n";
		double wasted_progress_sec = TMPI_Debug1(runid, kmax, wtime_running)/1000000;
		std::cout << "Walltime runid=" << runid << " halo=" << N << " " << wtime_running/1000000 << " seconds. Wasted progress took (sec)=" << wasted_progress_sec << std::endl; 
		//int numtasks = size/2; // because NUMTHREADS=1 NUMNODES=2
		//std::cout << "Walltime runid=" << runid << " tasks=" << numtasks << " " << wtime_running/1000000 << " seconds. Wasted progress took (sec)=" << wasted_progress_sec << std::endl;
		//std::cout << "Task2Task Isend took (sec):" << wtime_task2task_isend/1000000 << std::endl; 
	}

	free(message);
	free(message_recv);

	TMPI_Finalize();

	return 0;
}
