#!/bin/bash
#BSUB -n 48
#BSUB -J tmpi

# vim: set colorcolumn= :
shopt -s expand_aliases

#module load tinympi
source build/set-env.sh

ENDPOINTSPERNODE=2
NUMTHREADS=1
# the 2nd and 3rd members of $numtasks_ below should be adjusted so that their product equals NUMTHREADSxNUMTASKS
NUMNODES=2
NODE_GRID_X=2
NODE_GRID_Y=1
# NODE_GRIDX * NODE_GRID_Y must be = NUMNODES
HALO_DEPTH=5000

configurations_=()
for kmax_ in "10240" #"256" #"512" "1024" ## for rdv
# for kmax_ in "128" "256" "512" ### for eager
do
	for xysize_ in "64 128" #"128 128" "128 256" "256 256"
	do
		xsize=`echo $xysize_ | cut -d ' ' -f1`
		ysize=`echo $xysize_ | cut -d ' ' -f2`
		for numtasks_ in "01 1 1 8 8" "02 1 2 8 4" "04 2 2 4 4" "08 2 4 4 2"
		do
			numtasks=`echo $numtasks_ | cut -d ' ' -f1`
			# Task grid inside a Node = $locxproc x $locyproc
			locxproc=`echo $numtasks_ | cut -d ' ' -f2`
			locyproc=`echo $numtasks_ | cut -d ' ' -f3`
			numxrep=`echo $numtasks_ | cut -d ' ' -f4`
			numyrep=`echo $numtasks_ | cut -d ' ' -f5`
			xsz=`echo "$numxrep * $xsize " | bc`
			ysz=`echo "$numyrep * $ysize " | bc`
			configuration=" $ENDPOINTSPERNODE $numtasks $NUMTHREADS $NUMNODES $locxproc $locyproc $NODE_GRID_X $NODE_GRID_Y $xsz $ysz $HALO_DEPTH $kmax_"
			configurations_+=(" $configuration ")
			#echo $configuration
		done
	done
done
#exit
# # send ysize data to left / right and xsize data to top / bottom
# # top / bottom is usually slower because it is node2node
# #     endpoints/node numtasks  numthreads  numnodes #loc-xproc #loc-yproc #glob-xproc #glob-yproc xsize/proc  ysize/proc border  kmax
# declare -a configurations=( \
# 	"        2          01          8         6         4           2          2          3          512          512     1       16"
# 	"        2          02          8         6         4           4          2          3          512          256     1       16"
# 	"        2          04          8         6         8           4          2          3          256          256     1       16"
# 	"        2          08          8         6         8           8          2          3          256          128     1       16"
# 	"        2          16          8         6        16           8          2          3          128          128     1       16"
# 	"        2          32          8         6        16          16          2          3          128           64     1       16"
# 	"        2          64          8         6        32          16          2          3           64           64     1       16"
# )
cd build

for configuration in "${configurations_[@]}"
do

	folder=$(tr ' ' '_' <<< $configuration)
	if [ -n "$deleteall" ]; then
		rm -rf $folder
	elif [ -d "$folder" ]; then
		read -p "Would you like to delete $folder/*? [y/a/N] " doit
		case "$doit" in
			[yY])
				rm -rf $folder
				;;
			[aA])
				rm -rf $folder
				deleteall=1
				;;
			[n])
				continue;
				;;
			*)
				printf 'No files deleted\n'
				exit -1
				;;
		esac
	fi
	rm lsb.*
	eppernode="` sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f1`"
	numtasks="`  sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f2`"
	numthreads="`sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f3`"
	numnodes="`  sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f4`"
	locxpnum="`  sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f5`"
	locypnum="`  sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f6`"
	remxpnum="`  sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f7`"
	remypnum="`  sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f8`"
	xsize="`     sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f9`"
	ysize="`     sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f10`"
	border="`    sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f11`"
	kmax="`      sed 's/\s\+/ /g' <<<"$configuration" | sed 's/^\s*\(.*\)\s*$/\1/g' | cut -d ' ' -f12`"
	numtasks_=$(echo $numtasks | sed 's/^0*//')
	echo cmake -DTinyMPI_NUMBEROFTASKS:STRING=$numtasks_ -DTinyMPI_NUMBEROFTHREADS:STRING=$numthreads \
			-DTinyMPI_VERBSENDPOINTSPERNODE:STRING=$eppernode .
	echo "@ mpirun -bynode -np $numnodes ./benchmark $locxpnum $locypnum $remxpnum $remypnum $xsize $ysize $border $kmax"
	cmake -DTinyMPI_NUMBEROFTASKS:STRING=$numtasks_ -DTinyMPI_NUMBEROFTHREADS:STRING=$numthreads \
			-DTinyMPI_VERBSENDPOINTSPERNODE:STRING=$eppernode -DCMAKE_INSTALL_PREFIX:STRING=${HOME}/opt/tinympi -DCMAKE_BUILD_TYPE:STRING=Release .
	make -j12 benchmark

	mpirun -x LD_PRELOAD=./utils/libmalloc_overwrite.so -bynode -np $numnodes benchmark $locxpnum $locypnum $remxpnum $remypnum $xsize $ysize $border $kmax
	mkdir $folder
	mv lsb.* $folder

	cd $folder
	rm -f out
	rm -f out2
	touch out
	for lsbfile in lsb.tinympi.r*
	do
		paste out > out2 <(cat \
				<(echo "$lsbfile" | sed "s/lsb.tinympi.//") \
				<(cat $lsbfile | tail -n +12 | tr -s ' ' | cut -d ' ' -f 4) \
		)
		mv out2 out
	done
	cd ..
# 	mv $folder /scratch/mlukas

# 	rm rank*out
done

cd ..

echo Benchmark complete

