# Try to find Fabric headers and libraries.
#
# Usage of this module as follows:
#
#     find_package(Fabric)
#
# Variables used by this module, they can change the default behaviour and need
# to be set before calling find_package:
#
#  FABRIC_ROOT         Set this variable to the root installation of
#                      libpapi if the module has problems finding the
#                      proper installation path.
#
# Variables defined by this module:
#
#  FABRIC_LIBRARIES        The Fabric library
#  FABRIC_INCLUDE_DIRS     The location of Fabric headers

if (NOT FABRIC_ROOT AND DEFINED ENV{FABRIC_ROOT})
    set(FABRIC_ROOT "$ENV{FABRIC_ROOT}" CACHE PATH "Fabric base directory location (optional, used for nonstandard installation paths)")
    mark_as_advanced(FABRIC_ROOT)
endif()

if (FABRIC_ROOT)
    set(FABRIC_INCLUDE_PATH PATHS "${FABRIC_ROOT}/include" NO_DEFAULT_PATH)
    set(FABRIC_LIBRARY_PATH PATHS "${FABRIC_ROOT}/lib" NO_DEFAULT_PATH)
endif()

find_library(FABRIC_LIBRARIES
    NAMES fabric
    HINTS ${FABRIC_LIBRARY_PATH}
)

find_path(FABRIC_INCLUDE_DIRS
    NAMES rdma/fabric.h
    HINTS ${FABRIC_INCLUDE_PATH}
)

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(Fabric DEFAULT_MSG
    FABRIC_LIBRARIES
    FABRIC_INCLUDE_DIRS
)

mark_as_advanced(
    FABRIC_LIBRARIES
    FABRIC_INCLUDE_DIRS
)

