# Try to find SciBench headers and libraries.
#
# Usage of this module as follows:
#
#     find_package(SciBench)
#
# Variables used by this module, they can change the default behaviour and need
# to be set before calling find_package:
#
#  SCIBENCH_ROOT         Set this variable to the root installation of
#                      SciBench if the module has problems finding the
#                      proper installation path.
#
# Variables defined by this module:
#
#  SCIBENCH_LIBRARIES        The SciBench library
#  SCIBENCH_INCLUDE_DIRS     The location of SciBench headers

if (NOT SCIBENCH_ROOT AND DEFINED ENV{SCIBENCH_ROOT})
    set(SCIBENCH_ROOT "$ENV{SCIBENCH_ROOT}" CACHE PATH "SciBench base directory location (optional, used for nonstandard installation paths)")
    mark_as_advanced(SCIBENCH_ROOT)
endif()

if (SCIBENCH_ROOT)
    set(SCIBENCH_INCLUDE_PATH PATHS "${SCIBENCH_ROOT}/include" NO_DEFAULT_PATH)
    set(SCIBENCH_LIBRARY_PATH PATHS "${SCIBENCH_ROOT}/lib" NO_DEFAULT_PATH)
endif()

find_library(SCIBENCH_LIBRARIES
    NAMES lsb
    HINTS ${SCIBENCH_LIBRARY_PATH}
)

find_path(SCIBENCH_INCLUDE_DIRS
    NAMES liblsb.h
    HINTS ${SCIBENCH_INCLUDE_PATH}
)

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(SciBench DEFAULT_MSG
    SCIBENCH_LIBRARIES
    SCIBENCH_INCLUDE_DIRS
)

mark_as_advanced(
    SCIBENCH_LIBRARIES
    SCIBENCH_INCLUDE_DIRS
)

