#ifndef _PERF_TIMING_H_
#define _PERF_TIMING_H_

#include <vector>
#include <unistd.h>
#include <asm/unistd.h>
#include <linux/perf_event.h>
#include <sys/ioctl.h>
#include <sched.h>
#include <sys/mman.h>

#include <cerrno>
#include <cstring>

struct event_t {
	int fd;
	__u64 type;
	__u64 config;
};
inline std::vector<event_t>& get_events(int i) {
	static std::vector<event_t> events[4];
	if (events[i].size() == 0) {
		__u64 CACHE_FLAGS_AR = (PERF_COUNT_HW_CACHE_OP_READ << 8) | (PERF_COUNT_HW_CACHE_RESULT_ACCESS << 16);
		__u64 CACHE_FLAGS_MR = (PERF_COUNT_HW_CACHE_OP_READ << 8) | (PERF_COUNT_HW_CACHE_RESULT_MISS   << 16);
		__u64 CACHE_FLAGS_AW = (PERF_COUNT_HW_CACHE_OP_WRITE << 8) | (PERF_COUNT_HW_CACHE_RESULT_ACCESS << 16);
		__u64 CACHE_FLAGS_MW = (PERF_COUNT_HW_CACHE_OP_WRITE << 8) | (PERF_COUNT_HW_CACHE_RESULT_MISS   << 16);

		events[0].push_back({0, PERF_TYPE_HARDWARE, PERF_COUNT_HW_REF_CPU_CYCLES});
		events[0].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_LL  | CACHE_FLAGS_AR});
		events[0].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_LL  | CACHE_FLAGS_MR});

		events[1].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_L1D | CACHE_FLAGS_AW});
		events[1].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_L1D | CACHE_FLAGS_MW});

		events[2].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_LL  | CACHE_FLAGS_AW});
		events[2].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_LL  | CACHE_FLAGS_MW});

		events[3].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_L1D | CACHE_FLAGS_AR});
		events[3].push_back({0, PERF_TYPE_HW_CACHE, PERF_COUNT_HW_CACHE_L1D | CACHE_FLAGS_MR});
	}
	return events[i];
}

inline void perf_setup() {
	cpu_set_t set;
	CPU_ZERO(&set);
	CPU_SET(1, &set);
	sched_setaffinity(0, sizeof(cpu_set_t), &set);
	sched_param param;
	param.sched_priority = sched_get_priority_max(SCHED_FIFO);
	sched_setscheduler(0, SCHED_FIFO, &param);

	mlockall(MCL_CURRENT | MCL_FUTURE);
}

inline void perf_init(int i) {
	std::vector<event_t>& events = get_events(i);
	for (unsigned i = 0; i < events.size(); ++i) {
		std::clog << "register event: " << i << std::endl;
		perf_event_attr pe = {};
		pe.type = events[i].type;
		pe.size = sizeof(perf_event_attr);
		pe.config = events[i].config;
		pe.sample_freq = 1./10000000;
		pe.freq = 1;
		pe.disabled = i == 0 ? 1 : 0;
		pe.exclude_kernel = 1;
		pe.exclude_hv = 1;
		pe.exclude_idle = 1;
		events[i].fd = syscall(__NR_perf_event_open, &pe, 0, -1, i == 0 ? -1 : events[0].fd, 0);
		if (events[i].fd < 0) {
			std::cout << "an error occured: " << strerror(errno) << std::endl;
			exit(1);
		}
	}
}
inline void perf_start(int i) {
	std::vector<event_t>& events = get_events(i);
	ioctl(events[0].fd, PERF_EVENT_IOC_RESET);
	ioctl(events[0].fd, PERF_EVENT_IOC_ENABLE);
}
inline std::vector<__u64> perf_stop(int i) {
	std::vector<event_t>& events = get_events(i);
	ioctl(events[0].fd, PERF_EVENT_IOC_DISABLE);

	//std::vector<long long unsigned int> res(events.size());
	std::vector<__u64> res(events.size());
	int j = 0;
	for (auto& event : events) {
		read(event.fd, &res[j++], sizeof(__u64));
	}
	return res;
}


#endif //_PERF_TIMING_H_
