#define EXT_USESCIBENCH

#define MEASALL
#include <sstream>
#include <string>
#include <iomanip>
#include <cassert>


#include "./tinympi/tmpi.h"
#include <debug/debugger.h>

#include <unistd.h>
#include <sched.h>
#include <sys/mman.h>

int main(int argc, char** argv) {
	sched_param param;
	param.sched_priority = sched_get_priority_max(SCHED_FIFO);

	mlockall(MCL_CURRENT | MCL_FUTURE);

	TMPI_Init(&argc, &argv);
	const int length = 10000;
	int* message = new int[length];
	int* message_recv = new int[length];

	assert(sizeof(int) == 4);

	int rank, size, res;
	(void) res;
	res = TMPI_Comm_rank(TMPI_COMM_WORLD, &rank);
	res = TMPI_Comm_size(TMPI_COMM_WORLD, &size);
	TMPI_LOG("rank", rank);
	TMPI_LOG("size", size);

	if (rank == 0)
		tinympi::Debugger::color = ANSI_COLOR_RED;
	else if (rank == 1)
		tinympi::Debugger::color = ANSI_COLOR_BLUE;
	else if (rank == 2)
		tinympi::Debugger::color = ANSI_COLOR_GREEN;

	if (rank == 0)
		for (int i = 0; i < length; ++i)
			message[i] = 10*i + rank;
	else
		for (int i = 0; i < length; ++i)
			message[i] = -12;
	TMPI_Barrier(TMPI_COMM_WORLD);

	TMPI_Bcast(message, sizeof(int)*length, TMPI_BYTE, 0, TMPI_COMM_WORLD);

	message[0] = rank + 2 * (rank % 3);
	TMPI_Allreduce(message, message_recv, 1, TMPI_INT, TMPI_SUM, TMPI_COMM_WORLD);

	int expected_result = 0;
	for (int i = 0; i < size; ++i) {
		expected_result += (i + 2 * (i % 3));
	}
	using namespace tinympi;
	TMPI_LOG("done", "result = " << message_recv[0] << " (expected: " << expected_result << ")");
	TMPI_ASSERT_FATAL(expected_result == message_recv[0], "wrong result", __FILE__, __LINE__);


	int imax = 100;

	TMPI_Request treq[10];
	TMPI_Status tstatus[10];
	int dest = (rank + 1) % size;
	int src = (rank + size - 1) % size;
	TMPI_LOG_LEVEL(0, "src", src);
	TMPI_LOG_LEVEL(0, "dest", dest);
	TMPI_LOG_LEVEL(0, "my global rank", tinympi::tasks::get_globalunique_id());

	// Let's try a self send...
	for (int i = 0; i < 100; ++i) {
		message[i] = i;
		message_recv[i] = 0;
	}
	TMPI_Isend(message,      10, TMPI_INT, dest, 10, TMPI_COMM_WORLD, &treq[0]);
	TMPI_Irecv(message_recv, 20, TMPI_INT, TMPI_ANY_SOURCE, TMPI_ANY_TAG, TMPI_COMM_WORLD, &treq[1]);
	TMPI_Waitall(2, treq, TMPI_STATUSES_IGNORE);
	for (int i = 0; i < 10; ++i) {
		TMPI_ASSERT_FATAL(message[i] == i, "message[" << i << "] == " << message[i], __FILE__,
				__LINE__);
		TMPI_ASSERT_FATAL(message_recv[i] == i, "message_recv[" << i << "] == " << message_recv[i],
				__FILE__, __LINE__);
	}
	for (int i = 10; i < 100; ++i) {
		TMPI_ASSERT_FATAL(message[i] == i, "message[" << i << "] == " << message[i], __FILE__,
				__LINE__);
		TMPI_ASSERT_FATAL(message_recv[i] == 0, "message_recv[" << i << "] == " << message_recv[i],
				__FILE__, __LINE__);
	}

	// and other sends
	for (int i = 0; i < imax; ++i) {

		TMPI_LOG("starting iteration", i);

		if (rank == 0) {
			std::cout << i << std::endl;
			for (int j = 0; j < length; ++j)
				message[j] = j;

			TMPI_Isend(message, 3000, TMPI_INT, dest, 1, TMPI_COMM_WORLD, &treq[0]);
			TMPI_Isend(message+3000, 3000, TMPI_INT, dest, 2, TMPI_COMM_WORLD, &treq[1]);
			TMPI_Isend(message+6000, 3000, TMPI_INT, dest, 3, TMPI_COMM_WORLD, &treq[2]);
			TMPI_Isend(message+9000, 300, TMPI_INT, dest, 1, TMPI_COMM_WORLD, &treq[3]);
			TMPI_Isend(message+9300, 300, TMPI_INT, dest, 2, TMPI_COMM_WORLD, &treq[4]);
			TMPI_Isend(message+9600, 300, TMPI_INT, dest, 3, TMPI_COMM_WORLD, &treq[5]);
			TMPI_Isend(message+9900, 30, TMPI_INT, dest, 1, TMPI_COMM_WORLD, &treq[6]);
			TMPI_Isend(message+9930, 30, TMPI_INT, dest, 2, TMPI_COMM_WORLD, &treq[7]);
			TMPI_Isend(message+9960, 30, TMPI_INT, dest, 3, TMPI_COMM_WORLD, &treq[8]);
			TMPI_Isend(message+9990, 10, TMPI_INT, dest, 4, TMPI_COMM_WORLD, &treq[9]);
			TMPI_Waitall(3, &treq[0], &tstatus[0]);
			TMPI_Waitall(5, &treq[3], &tstatus[3]);
			TMPI_Waitall(2, &treq[8], &tstatus[8]);

			using namespace tinympi;
			TMPI_LOG("test", "go to recvs");

			TMPI_Irecv(message_recv, 3000, TMPI_INT, src, 1, TMPI_COMM_WORLD, &treq[0]);
			TMPI_Irecv(message_recv+9000, 300, TMPI_INT, src, 1, TMPI_COMM_WORLD, &treq[1]);
			TMPI_Waitall(2, &treq[0], &tstatus[0]);
			TMPI_Irecv(message_recv+9900, 30, TMPI_INT, src, 1, TMPI_COMM_WORLD, &treq[2]);
			TMPI_Irecv(message_recv+3000, 3000, TMPI_INT, src, 2, TMPI_COMM_WORLD, &treq[3]);
			TMPI_Irecv(message_recv+9990, 10, TMPI_INT, src, 4, TMPI_COMM_WORLD, &treq[4]);
			TMPI_Irecv(message_recv+6000, 3000, TMPI_INT, src, 3, TMPI_COMM_WORLD, &treq[5]);
			TMPI_Irecv(message_recv+9600, 300, TMPI_INT, src, 3, TMPI_COMM_WORLD, &treq[6]);
			TMPI_Waitall(2, &treq[2], &tstatus[2]);
			TMPI_Waitall(3, &treq[4], &tstatus[4]);
			TMPI_Irecv(message_recv+9960, 30, TMPI_INT, src, 3, TMPI_COMM_WORLD, &treq[7]);
			TMPI_Irecv(message_recv+9300, 300, TMPI_INT, src, 2, TMPI_COMM_WORLD, &treq[8]);
			TMPI_Irecv(message_recv+9930, 30, TMPI_INT, src, 2, TMPI_COMM_WORLD, &treq[9]);
			TMPI_Waitall(3, &treq[7], &tstatus[7]);

			for (int j = 0; j < length; ++j)
				if (message_recv[j] != (message[j] + (((j+1) % 8) + 1) * (size - 1)))
					std::cout << j << ": Expected " << (message[j] + (((j+1) % 8) + 1)*(size-1))
							<< ". Got " << message_recv[j] << std::endl;
		} else {
			TMPI_Irecv(message, 3000, TMPI_INT, src, 1, TMPI_COMM_WORLD, &treq[0]);
			TMPI_Irecv(message+9000, 300, TMPI_INT, src, 1, TMPI_COMM_WORLD, &treq[1]);
			TMPI_Waitall(2, &treq[0], &tstatus[0]);
			TMPI_Irecv(message+9900, 30, TMPI_INT, src, 1, TMPI_COMM_WORLD, &treq[2]);
			TMPI_Irecv(message+3000, 3000, TMPI_INT, src, 2, TMPI_COMM_WORLD, &treq[3]);
			TMPI_Irecv(message+9990, 10, TMPI_INT, src, 4, TMPI_COMM_WORLD, &treq[4]);
			TMPI_Irecv(message+6000, 3000, TMPI_INT, src, 3, TMPI_COMM_WORLD, &treq[5]);
			TMPI_Irecv(message+9600, 300, TMPI_INT, src, 3, TMPI_COMM_WORLD, &treq[6]);
			TMPI_Waitall(2, &treq[2], &tstatus[2]);
			TMPI_Waitall(3, &treq[4], &tstatus[4]);
			TMPI_Irecv(message+9960, 30, TMPI_INT, src, 3, TMPI_COMM_WORLD, &treq[7]);
			TMPI_Irecv(message+9300, 300, TMPI_INT, src, 2, TMPI_COMM_WORLD, &treq[8]);
			TMPI_Irecv(message+9930, 30, TMPI_INT, src, 2, TMPI_COMM_WORLD, &treq[9]);
			TMPI_Waitall(3, &treq[7], &tstatus[7]);

			using namespace tinympi;
			TMPI_LOG("test", "go to sends");

			for (int j = 0; j < length; ++j)
				message[j] = message[j] + ((j+1) % 8) + 1;

			TMPI_Isend(message, 3000, TMPI_INT, dest, 1, TMPI_COMM_WORLD, &treq[0]);
			TMPI_Isend(message+3000, 3000, TMPI_INT, dest, 2, TMPI_COMM_WORLD, &treq[1]);
			TMPI_Isend(message+6000, 3000, TMPI_INT, dest, 3, TMPI_COMM_WORLD, &treq[2]);
			TMPI_Isend(message+9000, 300, TMPI_INT, dest, 1, TMPI_COMM_WORLD, &treq[3]);
			TMPI_Isend(message+9300, 300, TMPI_INT, dest, 2, TMPI_COMM_WORLD, &treq[4]);
			TMPI_Isend(message+9600, 300, TMPI_INT, dest, 3, TMPI_COMM_WORLD, &treq[5]);
			TMPI_Waitall(3, &treq[0], &tstatus[0]);
			TMPI_Isend(message+9900, 30, TMPI_INT, dest, 1, TMPI_COMM_WORLD, &treq[6]);
			TMPI_Isend(message+9930, 30, TMPI_INT, dest, 2, TMPI_COMM_WORLD, &treq[7]);
			TMPI_Isend(message+9960, 30, TMPI_INT, dest, 3, TMPI_COMM_WORLD, &treq[8]);
			TMPI_Isend(message+9990, 10, TMPI_INT, dest, 4, TMPI_COMM_WORLD, &treq[9]);
			TMPI_Waitall(5, &treq[3], &tstatus[3]);
			TMPI_Waitall(2, &treq[8], &tstatus[8]);
		}

	}

	delete[] message;
	delete[] message_recv;

	TMPI_Finalize();
	TMPI_LOG("I finished....","");
}

	/**/
