// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#include "global.h"
#include "pt2pt.h"
#include "debug/debugger.h"

namespace tinympi {
namespace collectives {
namespace impl {
	using namespace tinympi::impl;

/////////////////////////////////////
// barrier
namespace barrier {

inline int barrier_impl(const Communicator* comm) {
	int res;

	rank_type my_rank = comm->get_rank();
	rank_type size = comm->get_size();

	if (size == 0)
		return TMPI_SUCCESS;

	int mask = 1;
	while (mask < size) {
		int dest = (my_rank + mask) % size;
		int src = (my_rank - mask + size) % size;
		res = pt2pt::sendrecv(nullptr, 0, dest, TMPI_BARRIER_TAG, nullptr, 0, src, TMPI_BARRIER_TAG,
				comm->get_id(), comm, TMPI_STATUS_IGNORE);
		TMPI_ASSERT_FATAL(res == TMPI_SUCCESS, "sendrecv() failed", __FILE__, __LINE__);

		mask <<= 1;
	}

	return TMPI_SUCCESS;
}

} // END barrier


}
}
}
