// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#include "debug/debugger.h"
#include "threads.h"
#include "tasks.h"
#include "utility/external.h"

#include <cstdlib>

namespace perfdbg {
	thread_local double wtime_isend_internal = 0;
	thread_local unsigned int verbs_isend_count = 0;
	thread_local double time_TMPI_Waitall, time_TMPI_Waitall_start, time_TMPI_Waitall_end;
	thread_local double time_TMPI_Isend, time_TMPI_Irecv;

}

namespace tinympi {
	thread_local std::string Debugger::color = "";
// 	std::ostream& Debugger::os = std::cout;

	thread_local std::ofstream Debugger::of { };

	void preinit_debugger() {
		int node_rank = utility::external::get_noderank();
		system((std::string("rm rank") + std::to_string(node_rank) + std::string("-*.out")).c_str());
		tinympi::Debugger::of.open(std::string("rank") + std::to_string(node_rank) + "-preinit.out");
	}
	void postinit_debugger() {
		int node_rank = utility::external::get_noderank();
		int thread_rank = env::thread_id;

		if (tinympi::Debugger::of.is_open())
			tinympi::Debugger::of.close();

		tinympi::Debugger::of.open(std::string("rank") + std::to_string(node_rank) + "-"
				+ std::to_string(thread_rank) + ".out", std::ofstream::app);
	}

	void Debugger::log(std::string text, std::string color, std::ofstream& of) {
		int task_rank;
		if (tasks::get_current() != nullptr)
			task_rank = tasks::get_current()->get_identifier().local.task_id;
		else
			task_rank = -1;
		of << color << std::left << std::setw(4) << task_rank << text << ANSI_COLOR_RESET;
		of.flush();
	}
	void Debugger::log(std::string text, std::ofstream& of) {
		int task_rank;
		if (tasks::get_current() != nullptr)
			task_rank = tasks::get_current()->get_identifier().local.task_id;
		else
			task_rank = -1;
		of << std::left << std::setw(4) << task_rank << text;
		of.flush();
	}
}
