// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _TMPI_DEBUG_H_635106f3226c46d3d114158785752466
#define _TMPI_DEBUG_H_635106f3226c46d3d114158785752466

#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>

#include <fstream>

#define DEBUG
#define TMPI_USE_LOG_LEVEL (7)

#undef DEBUG
// #define NO_ASSERT_FATAL

// #define DOSTATISTICS

namespace perfdbg { // vars for performance debugging
	extern thread_local double wtime_isend_internal;
	extern thread_local unsigned int verbs_isend_count;
	extern thread_local double time_TMPI_Waitall, time_TMPI_Waitall_start, time_TMPI_Waitall_end;
	extern thread_local double time_TMPI_Isend, time_TMPI_Irecv;
}


namespace tinympi {

#define ANSI_COLOR_RED     "\x1b[31m"
#define ANSI_COLOR_GREEN   "\x1b[32m"
#define ANSI_COLOR_YELLOW  "\x1b[33m"
#define ANSI_COLOR_BLUE    "\x1b[34m"
#define ANSI_COLOR_MAGENTA "\x1b[35m"
#define ANSI_COLOR_CYAN    "\x1b[36m"
#define ANSI_COLOR_RESET   "\x1b[0m"
#define FAILED_STRING "\x1b[1mERROR: "

class Debugger {
public:
	static void log(std::string text, std::string color, std::ofstream& of = Debugger::of);
	static void log(std::string text, std::ofstream& of = Debugger::of);
	static thread_local std::string color;
	static thread_local std::ofstream of;

};

#ifdef DOSTATISTICS
#define STATISTICS_ADD(dest, sendlength) \
	{ \
		tinympi::Debugger::log(std::to_string(dest) + " " + std::to_string(sendlength) + "\n"); \
	}
#else
#define STATISTICS_ADD(dest, sendlength) {}
#endif


#ifdef DEBUG
#define TMPI_LOG(description, value) \
	{ \
		std::stringstream ss1; \
		ss1 << description; \
		std::stringstream ss; \
		ss << std::left << std::setw(30) << ss1.str() << value << "\n"; \
		tinympi::Debugger::log(ss.str()); \
	}
#define TMPI_LOG_LEVEL(level, description, value) \
	{ \
		if ((int)level <= (int)TMPI_USE_LOG_LEVEL) { \
			TMPI_LOG(description, value) \
		} \
	}
#else
#define TMPI_LOG(d,v) {}
#define TMPI_LOG_LEVEL(l,d,v) {}
#endif

#define TMPI_COUT(text) std::cout<<text<<std::endl;std::cout.flush();

#define TMPI_LOG_EMPTY() TMPI_LOG("", "")

void postinit_debugger();
void preinit_debugger();
}

#endif //_TMPI_DEBUG_H_
