// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _GLOBAL_H_635106f3226c46d3d114158785752466
#define _GLOBAL_H_635106f3226c46d3d114158785752466

#include "scibench.h"
#include "datatypes.h"

#define TMPI_SUCCESS 0
#define TMPI_ERROR -1

enum TMPI_Op {
	TMPI_MAX,
	TMPI_MIN,
	TMPI_SUM,
	TMPI_PROD,
	TMPI_LAND,
	TMPI_BAND,
	TMPI_LOR,
	TMPI_BOR,
	TMPI_LXOR,
	TMPI_BXOR,
	TMPI_MAXLOC,
	TMPI_MINLOC
};

#define TMPI_PROC_NULL -2

#define PROVIDER_VERBS 1
#define PROVIDER_OFI 2

#define PROVIDER PROVIDER_VERBS


namespace tinympi {
	// constexpr max does not exist <= C++11! for C++14 we may want to delete this.
	template<typename T> constexpr
	T const& max(T const& a, T const& b) {
		return a > b ? a : b;
	}

	// this is the rank of a processor. Should be signed because we use negative ranks, e.g., for
	// TMPI_NULL_PROC. They are used in communicators
	using rank_type   = int64_t;

	// types used to identify a task on a threaad, a thread on a node, or a node globally
	using global_id_t = uint64_t;
	using node_id_t   = uint32_t; constexpr unsigned node_id_bits   = 32;
	using thread_id_t = uint8_t;  constexpr unsigned thread_id_bits =  8;
	using task_id_t   = uint32_t; constexpr unsigned task_id_bits   = 20;
	// the local rank should be able to contain both thread_id and task_id
	using local_id_t  = uint32_t; constexpr unsigned local_id_bits= (thread_id_bits + task_id_bits);

	// type used for the context id (contains for example the communicator)
	using context_id_t= uint16_t; constexpr unsigned context_id_bits= 12;
	// type of the communicator id
	using comm_id_t   = uint16_t;

	// type used for the tag (allow negative types; used for collectives and ANY_TAG)
	using tag_t       = int32_t;  constexpr unsigned tag_bits       = 24;

	// used for the size of a send buffer
	using buflen_t    = int32_t;

	// used for the credit count in the scheduling
	using credit_t    = int16_t;

	constexpr tag_t TMPI_BARRIER_TAG = -1000;
	constexpr tag_t TMPI_ALLREDUCE_TAG = -1001;
	constexpr tag_t TMPI_BCAST_TAG = -1002;

	// each main thread should have a reasonably large stack
	constexpr unsigned stack_size = 2048*1024;

	namespace env {
		extern node_id_t num_nodes;
		extern thread_id_t num_threads;
		extern task_id_t num_tasks;
		const node_id_t num_cores = 24;

		extern node_id_t node_id;
		extern thread_local thread_id_t thread_id;
	}
}

#define TMPI_ANY_SOURCE -1
#define TMPI_ANY_TAG -1

#define TMPI_ALWAYSINLINE __attribute__((always_inline))

#endif //_GLOBAL_H
