// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _OFI_REQUEST_H_635106f3226c46d3d114158785752466
#define _OFI_REQUEST_H_635106f3226c46d3d114158785752466

#include <rdma/fabric.h>

#include "provider_impl/baserequest.h"

namespace tinympi {
namespace ofi {
namespace impl {

using namespace tinympi::impl;

//tinympi::ofi::impl::ofi_request_types
enum class ofi_request_types {
	OFI_SEND,
	OFI_RECV
};

#define OFI_REQUEST_MEMBERS \
	BASE_REQUEST_MEMBERS \
	ofi_request_types ofi_request_type; \
	OfiEndpoint* endpoint; \
	fi_context fabric_context;

//tinympi::ofi::impl::OfiRequest
struct OfiRequest {
	OFI_REQUEST_MEMBERS
};
REQUESTS_CONVERTIBLE(OfiRequest, BaseRequest)

static_assert(std::is_standard_layout<OfiRequest>::value, "OfiRequest must be POD!");
ASSERT_BASE_REQUEST_COMPATIBLE(OfiRequest)
#define ASSERT_OFI_REQUEST_COMPATIBLE(TYPE) \
	ASSERT_BASE_REQUEST_COMPATIBLE(TYPE) \
	VERIFY_COMPATIBILITY(OfiRequest, TYPE, ofi_request_type) \
	VERIFY_COMPATIBILITY(OfiRequest, TYPE, endpoint) \
	VERIFY_COMPATIBILITY(OfiRequest, TYPE, fabric_context)

namespace ofiRequest {
	inline void init(OfiRequest* req, ofi_request_types type) {
		baseRequest::init(reinterpret_cast<BaseRequest*>(req), provider_types::node2node);
		req->ofi_request_type = type;
	}

	// we have 64 bits available where we need to encode tag, dest and context (i.e. communicator,
	// send context), as well as the destination. The maximum tag must be at least 32767. Therefore
	// we will use at least 16 bits for the tag. In fact we use 24 bits for the tag, 28 for the
	// local destination rank (task / thread id) and 12 bits for the contextid
	// xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
	// ^<-  CID  -><-             DESTINATION   ->^ ^<-         TAG        ->^
	//             ^ THREAD ^^<-      TASK      ->^

	inline uint64_t get_tag(context_id_t cid, local_id_t dest, tag_t tag) {
		uint64_t res = cid << local_id_bits;
		res = (res | dest) << tag_bits;
		res = res | tag;
		return res;
	}
	inline uint64_t get_mask(tag_t tag) {
		uint64_t res = 0;
		if (tag == (tag_t) TMPI_ANY_TAG)
			res += ((uint64_t(1) << tag_bits) - 1);

		return res;
	}

	inline tag_t extract_tag(uint64_t send_tag) {
		return send_tag & ((uint64_t(1) << tag_bits) - 1);
	}
	inline local_id_t extract_dest(uint64_t send_tag) {
		return send_tag & (((uint64_t(1) << (local_id_bits + tag_bits))
				- (uint64_t(1) << tag_bits)) >> tag_bits);
	}
	inline context_id_t extract_contextid(uint64_t send_tag) {
		return send_tag & (!((uint64_t(1) << (local_id_bits + tag_bits)) - 1));
	}
	// from ofi we get a pointer to fi_context. We need to find the request. This is
	// is one of the reasons why we require allrequests to be POD.
	inline OfiRequest* from_context(fi_context* context) {
		return reinterpret_cast<OfiRequest*>(reinterpret_cast<uint8_t*>(context)
				- offsetof(OfiRequest, fabric_context));
	}
}

}
}
}

#endif //_OFI_REQUEST_H_
