// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _VERBS_ENDPOINT_H_635106f3226c46d3d114158785752466
#define _VERBS_ENDPOINT_H_635106f3226c46d3d114158785752466

#include <infiniband/verbs.h>
#include <array>
#include <list>
#include <cstring>
#include <mutex>
#include <atomic>

#include "verbs_eagerendpoint.h"
#include "verbs_rdvendpoint.h"

namespace tinympi {
namespace verbs {
namespace impl {

using namespace tinympi::impl;

class VerbsRecvRequest;
class VerbsSendRequest;
struct WaitingMessage {
	int tag;
	VerbsRecvRequest* recvrequest;
};

//tinympi::verbs::impl::VerbsEndpoint
struct VerbsEndpoint {
public:
	void setup(ibv_cq* cq, ibv_qp* qp, ibv_pd* pd);
	void destroy();

	ibv_cq* get_cq() const { return completion_queue_; }
	ibv_qp* get_qp() const { return queue_pair_; }
	rid_t get_send_rid() { return send_rid_cnt_++; }
	void post_dummy_recv() {
		TMPI_LOG_LEVEL(3, "VerbsEndpoint", "post dummy receive");

		int res;
		res = ibv_post_recv(get_qp(), &dummy_recv_, &dummy_bad_recv_);
		TMPI_ASSERT_FATAL(res == 0, "ibv_post_recv() returned " << res, __FILE__, __LINE__);

#ifdef DEBUG
		outstanding_recv_count++;
#endif
	}
	rid_t get_expected_rid() const { return expected_rid_; }
	rid_t get_recv_rid() { return expected_rid_++; }

#ifdef DEBUG
	void recvd_postwc() {
		outstanding_recv_count--;
	}
#endif

public:
	VerbsEagerEndpoint eager;
	VerbsRdvEndpoint rdv;
	std::mutex lock;

private:
	ibv_cq* completion_queue_;
	ibv_qp* queue_pair_;

	rid_t send_rid_cnt_;
	std::atomic<rid_t> expected_rid_;

	ibv_recv_wr dummy_recv_;
	ibv_recv_wr* dummy_bad_recv_;

#ifdef DEBUG
	int outstanding_recv_count;
#endif

};

inline void VerbsEndpoint::setup(ibv_cq* cq, ibv_qp* qp, ibv_pd* pd) {
	completion_queue_ = cq;
	queue_pair_ = qp;

	send_rid_cnt_ = 0;
	expected_rid_ = 0;

	std::memset(&dummy_recv_, 0, sizeof(ibv_recv_wr));
	dummy_recv_.wr_id = 0;

	eager.setup(pd);
	rdv.setup(pd);

#ifdef DEBUG
	outstanding_recv_count = 0;
#endif
}
inline void VerbsEndpoint::destroy() {
#ifdef DEBUG
	TMPI_DASSERT_FATAL(outstanding_recv_count == 0,
			"outstanding recv posted (" << outstanding_recv_count << ")", __FILE__, __LINE__);
#endif

	eager.destroy();
	rdv.destroy();
}

}
}
}

#endif //_VERBS_ENDPOINT_H_
