// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _VERBS_MRU_H_635106f3226c46d3d114158785752466
#define _VERBS_MRU_H_635106f3226c46d3d114158785752466

// adapted from: http://www.boost.org/doc/libs/1_61_0/libs/multi_index/example/serialization.cpp
#include <boost/multi_index_container.hpp>
#include <boost/multi_index/hashed_index.hpp>
#include <boost/multi_index/identity.hpp>
#include <boost/multi_index/sequenced_index.hpp>

#include "global.h"

#include "verbs_provider.h"

namespace tinympi {
namespace verbs {

namespace impl {
class VerbsEndpoint;
}

namespace mrulist {
namespace impl {
	using namespace tinympi::verbs::impl;

// most recently used verbs endpoints
class VerbsEndpointMRUList {
private:
	using item_type = VerbsEndpoint*;
	using item_list = boost::multi_index::multi_index_container<
		item_type,
		boost::multi_index::indexed_by<
			boost::multi_index::sequenced<>,
			boost::multi_index::hashed_unique<boost::multi_index::identity<item_type> >
		>
	>;

public:

	inline void prioritize(const item_type& item) {
		auto it = il.get<1>().find(item);
		il.relocate(il.begin(), il.project<0>(it));
	}
	inline item_type get_front() {
		item_type front = il.front();
		il.relocate(il.end(), il.begin());
		return front;
	}
	inline void init(VerbsEndpoint** endpoints, global_id_t num_endpoints) {
		for (global_id_t i = 0; i < num_endpoints; ++i) {
			if (endpoints[i] != nullptr)
				il.push_back(endpoints[i]);
		}
	}

private:
	item_list il;

};
extern thread_local VerbsEndpointMRUList endpoint_mrulist_;
} // END NAMESPACE impl

inline tinympi::verbs::impl::VerbsEndpoint* get_endpoint() {
	using namespace impl;
	return endpoint_mrulist_.get_front();
}
inline void prioritize_endpoint(impl::VerbsEndpoint* ep) {
	using namespace impl;
	endpoint_mrulist_.prioritize(ep);
}
inline void init(impl::VerbsEndpoint** endpoint, global_id_t num_endpoints) {
	using namespace impl;
	endpoint_mrulist_.init(endpoint, num_endpoints);
}

} // END NAMESPACE mrulist
} // END NAMESPACE verbs
} // END NAMESPACE tinympi

#endif //_VERBS_MRU_H_

