// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _VERBS_UNKNOWNREQUEST_H_635106f3226c46d3d114158785752466
#define _VERBS_UNKNOWNREQUEST_H_635106f3226c46d3d114158785752466

#include "verbs_request.h"

namespace tinympi {
namespace verbs {
namespace impl {

#define VERBS_UNKNOWNREQUEST_MEMBERS \
	VERBS_REQUEST_MEMBERS

//tinympi::verbs::impl::VerbsRequest
struct VerbsUnknownRequest {
	VERBS_UNKNOWNREQUEST_MEMBERS
};
static_assert(std::is_standard_layout<VerbsUnknownRequest>::value, "VerbsUnknownRequest must be POD!");

REQUESTS_CONVERTIBLE(VerbsUnknownRequest, VerbsRequest)
REQUESTS_CONVERTIBLE(VerbsUnknownRequest, BaseRequest)

namespace verbsUnknownRequest {
	inline void init(VerbsUnknownRequest* req) {
		// no lock required
		verbsRequest::init(TO_VerbsRequest(req), verbs_request_types::VERBS_UNKNOWN);
	}

	inline VerbsUnknownRequest* generate_anysource_waiting_request(tag_t tag,
			context_id_t context_id, void* buf, buflen_t buflen) {
		// no lock required

		VerbsUnknownRequest* req = TO_VerbsUnknownRequest(baseRequest::malloc());
		init(req);

		// fill request
		INIT_BASE_REQUEST(req, \
			TMPI_ANY_SOURCE, \
			context_id, \
			tag, \
			buf, \
			buflen, \
			tasks::get_current() \
		)
		TMPI_LOG_LEVEL(2,"UnknownRequest", "generate (" << req << ", size = " << buflen << ")");

		return req;
	}

	inline VerbsUnknownRequest* generate_waiting_request(const tasks::Task& pos, tag_t tag,
			context_id_t context_id, void* buf, buflen_t buflen) {
		// no lock required
		VerbsUnknownRequest* req = TO_VerbsUnknownRequest(baseRequest::malloc());
		init(req);

		// fill request
		INIT_BASE_REQUEST(req, \
			pos.get_identifier().global_identifier, \
			context_id, \
			tag, \
			buf, \
			buflen, \
			tasks::get_current() \
		)
		TMPI_LOG_LEVEL(2,"UnknownRequest", "generate (" << req << ", size = " << buflen << ")");

		return req;
	}

	inline bool fill_status(VerbsUnknownRequest*, TMPI_Status*) {
		// no lock required
		return false;
	}

	inline int try_cancel(VerbsUnknownRequest* req) {
		// LOCK REQUIRED FOR node queue (OK)

		// An unknown request must be in the waiting queue and nowhere else

		// we first look in the waiting queue, because ANY SOURCE requests will be found there
		if (recvqueues::remove_waiting_request(TO_VerbsRequest(req))) {
			// we have successfully cancelled the request
			baseRequest::request_cancelled(TO_BaseRequest(req));
			return true;

		} else {
			// otherwise I don't know where to find it ...
			TMPI_FATAL_ERROR("cannot find request", __FILE__, __LINE__);
		}
	}
} // END NAMESPACE verbsUnknownRequest

} // END NAMESPACE impl
} // END NAMESPACE verbs
} // END NAMESPACE tinympi

#endif //_VERBS_REQUEST_H_
