// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _THREAD_SENDREQUEST_H_635106f3226c46d3d114158785752466
#define _THREAD_SENDREQUEST_H_635106f3226c46d3d114158785752466

#include "utility/error.h"

#include "thread_request.h"

namespace tinympi {
namespace thread2thread {
namespace impl {

#define THREAD_SEND_REQUEST_MEMBERS \
	THREAD_REQUEST_MEMBERS

//tinympi::thread::impl::ThreadSendRequest
struct ThreadSendRequest {
	THREAD_SEND_REQUEST_MEMBERS
};
static_assert(std::is_standard_layout<ThreadSendRequest>::value, "ThreadSendRequest must be POD!");

REQUESTS_CONVERTIBLE(ThreadSendRequest, ThreadRequest)
REQUESTS_CONVERTIBLE(ThreadSendRequest, BaseRequest)

ASSERT_THREAD_REQUEST_COMPATIBLE(ThreadRequest)

#define ASSERT_THREAD_SENDREQUEST_COMPATIBLE(TYPE) \
	ASSERT_THREAD_REQUEST_COMPATIBLE(TYPE)

namespace threadSendRequest {

	namespace impl {
		inline void init(ThreadSendRequest* req) {
			// no lock required
			threadRequest::init(TO_ThreadRequest(req), thread_request_types::THREAD_SEND);
		}
	}

	inline bool fill_status(ThreadSendRequest*, TMPI_Status*) {
		// no lock required
		return false;
	}
	// a thread request is fully trivial and only has the necessary information filled
	inline ThreadSendRequest* generate(void* buf, buflen_t buflen, tag_t tag,
			context_id_t context_id, const tasks::Task& dest_pos) {
		// NO LOCK required

		using namespace impl;

		ThreadSendRequest* req = TO_ThreadSendRequest(baseRequest::malloc());
		TMPI_LOG_LEVEL(3, "ThreadSendRequest", "generate request (req = " << req << ")");
		init(req);
		INIT_BASE_REQUEST( \
			req, \
			dest_pos.get_identifier().global_identifier, \
			context_id, \
			tag, \
			buf, \
			buflen, \
			tasks::get_current()
		)
		return req;
	}

} // END NAMESPACE threadSendRequest
} // END NAMESPACE impl
} // END NAMESPACE thread2thread
} // END NAMESPACE tinympi

#endif //_THREAD_SENDREQUEST_H_
