// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _TASKS_H_635106f3226c46d3d114158785752466
#define _TASKS_H_635106f3226c46d3d114158785752466

#include "global.h"

#include "tasks_impl/init.h"
#include "tasks_impl/stack.h"
#include "tasks_impl/scheduler.h"
#include "tasks_impl/task.h"
#include "tasks_impl/local_task.h"

namespace tinympi {
namespace tasks {

inline bool has_active_task() {
	return impl::scheduler_.has_active_task();
}

inline LocalTask* get_current() {
	return LocalTask::get_current_task();
}

template<typename keep_active>
inline int yield() {
	using namespace impl;

	// get a new task
	LocalTask* t = scheduler_.get_task();

	if (t != nullptr) {
		LSB_RESET();

		// this may also return the current task (I set my credit, call yield; in the meantime,
		// my data arrives, credit goes to 0 and task is added to scheduler)
		if (t == LocalTask::get_current_task()) {
			return 1;
		} else {
			// add this task to scheduler (depending on keep_active it is added to the active queue)
			scheduler_.yield_task<keep_active>();

			// then activate this task
			LocalTask::run_task(t);
			LSB_RECORD(TASKS_YIELD_RETURNED);
			return 1;
		}
	}
	return 0;
}
inline LocalTask& get_task(local_id_t taskid) {
	union {
		struct {
			task_id_t   task_id   : task_id_bits;
			uint8_t               : (32-task_id_bits-thread_id_bits);
			thread_id_t thread_id : thread_id_bits;
		} local;
		uint32_t id;
	} taskid_;
	taskid_.id = taskid;
	return impl::all_tasks_[taskid_.local.thread_id * env::num_tasks + taskid_.local.task_id];
}

inline void set_credit(credit_t credit) {
	using namespace impl;
	get_current()->set_credit(credit);
}
inline credit_t dec_credit(credit_t credit) {
	using namespace impl;
	return get_current()->dec_credit(credit, false);
}

inline global_id_t get_globalunique_id() {
	return get_current()->get_identifier().global_identifier;
}

inline local_id_t get_threadunique_id(global_id_t rank) {
	task_identifier ti;
	ti.global_identifier = rank;
	return ti.nodelevel.local_id;
}
inline local_id_t get_threadunique_id(const Task& task) {
	return get_threadunique_id(task.get_identifier().global_identifier);
}
inline local_id_t get_threadunique_id() {
	return get_threadunique_id(*get_current());
}
inline task_id_t get_taskunique_id() {
	return get_current()->get_identifier().local.task_id;
}

} // END NAMESPACE tasks
} // END NAMESPACE tinympi

#endif // _TASKS_H_
