#define SAVE_REGISTERS_TO_MEM(reg) \
	fnstcw (reg); \
	stmxcsr 4(reg); \
	mov %r15,  8(reg); \
	mov %r14, 16(reg); \
	mov %r13, 24(reg); \
	mov %r12, 32(reg); \
	mov %rbx, 40(reg); \
	mov %rbp, 48(reg); \
	mov %rdi, 56(reg);

#define RESTORE_REGISTERS_FROM_MEM(reg) \
	fldcw (reg); \
	ldmxcsr 4(reg); \
	mov 8(reg), %r15; \
	mov 16(reg), %r14; \
	mov 24(reg), %r13; \
	mov 32(reg), %r12; \
	mov 40(reg), %rbx; \
	mov 48(reg), %rbp; \
	mov 56(reg), %rdi;

#define SWAP(a, b, tmp) \
	mov a, tmp; \
	mov b, a; \
	mov tmp, b

.globl _copytaskstack
_copytaskstack:
	// just for this function we would not need to store the registers, but if we restore the stack
	// on another core, we will need to restore all necessary registers

	// rdi := state array
	// rsi := store address (to be stored)
	// rdx := return address (to be stored)
	// rcx := stack pointer (to be stored)
	// r8  := stack_top

	// get return address (on top of the stack) and store to rdx
	pop (%rdx)
	// store current stack pointer
	mov %rsp, (%rcx)

	// store the current registers to the address given (rdx)
	SAVE_REGISTERS_TO_MEM(%rsi)

	// %r8  := size of area
	mov %r8, %r14
	sub %rsp, %r8

	// state_array[0] := size of area
	mov %r8, (%rdi)
	add $8, %rdi

	// %rax := counter
	mov %rsp, %rax
copyloop:
	// state_array[i+1] := stack[i]
	mov (%rax), %r12
	mov %r12, (%rdi)
	// i += 1
	add $8, %rax
	add $8, %rdi

	// repeat as long below top
	cmp %rax, %r14
	jg copyloop

	// finally restore the registers from memory
	RESTORE_REGISTERS_FROM_MEM(%rsi)
	push (%rdx)

	// set our return value. This is necessary because we will return "at this place" also from
	// restorestate, but there we will return a non-nullptr
	mov $0, %rax

	ret $0

// activates another task
.globl _gototask
_gototask:
	// rdi := old stack pointer (to be stored)
	// rsi := new stack pointer (to be loaded)
	// rdx := store address
	// rcx := load address
	// r8  := return address (to be stored)
	// r9  := return address (to be loaded)

	// get return address (on top of the stack) and store to r8
	pop (%r8)
	// store current stack pointer
	mov %rsp, (%rdi)
	// restore stack pointer of the next task
	mov (%rsi), %rsp

	// store the current registers to the address given (rdx)
	SAVE_REGISTERS_TO_MEM(%rdx)
	// restore the registers of the other tasks from given address (rcx)
	RESTORE_REGISTERS_FROM_MEM(%rcx)

	// set a return value
	mov $1, %rax

	// return to the return address given in r9
	jmp *(%r9)
