// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _TASKS_TASK_H_635106f3226c46d3d114158785752466
#define _TASKS_TASK_H_635106f3226c46d3d114158785752466

#include <cassert>
#include <malloc.h>
#include <cstring>

#include "debug/debugger.h"
#include "global.h"

#include "task_global.h"
#include "task_identifier.h"
#include "stack.h"

namespace tinympi {

namespace tasks {

// the task identifier is used to uniquely identify each task. The sizes of task_id and thread_id
// are fixed with fabric tags
union task_identifier {
	struct {
		task_id_t   task_id   : task_id_bits;
		uint8_t               : (32-task_id_bits-thread_id_bits);
		thread_id_t thread_id : thread_id_bits;
		node_id_t   node_id   : node_id_bits;
	} local;
	struct {
		local_id_t local_id   : local_id_bits;
		node_id_t  node_id    : node_id_bits;
	} nodelevel;
	uint64_t global_identifier;
};
static_assert(sizeof(task_identifier) == 8, "unexpected padding for task_identifier detected");
static_assert(task_id_bits + thread_id_bits < 32 && node_id_bits == 32, "unexpected id-sizes");

class Task {
////////////////////////////
// VARIABLES
protected:
	task_identifier                   identifier_;
public:
	// this functions refer to the correct functions depending on whether tasks are on the same node
	// or not
	impl::isend_function_t            isend;
	impl::post_knownsource_recv_req_t post_knownsource_recv_request;

////////////////////////////
// FUNCTIONS
public:
	Task() {
	}
	Task(node_id_t node_id, thread_id_t thread_id, task_id_t task_id)
			: identifier_({task_id, thread_id, node_id}) {
	}
	const task_identifier& get_identifier() const {return identifier_; }
};

} // END NAMESPACE tasks
} // END NAMESPACE tinympi

#endif // _TASKS_TASK_H_
