// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _THREADS_ENV_H_635106f3226c46d3d114158785752466
#define _THREADS_ENV_H_635106f3226c46d3d114158785752466

#include <vector>

#include "barrier.h"

namespace tinympi {
namespace threads {
namespace impl {

	// thread_t stores information about each thread. With information about each started thread,
	// we are able to identify all threads, and to terminate each thread
	struct thread_t {
		thread_id_t rank;

		// information necessary to terminate the thread
		pthread_t thread;
		void* stack;
		void* stack_top;
		void* original_stackptr;
	};
	extern std::vector<thread_t> threads;
	extern thread_id_t num_threads_;

	// one thread context is transferred from the main thread to all new threads. It stores all
	// information for the new thread to startup and to detect its own identity
	struct thread_context {
		thread_t* my_thread;
		void* state_copy;
	};
	// each thread stores a pointer to its thread locally
	extern thread_local thread_t* self;

	// barrier for synchronization
	extern Barrier barrier_;

} // END NAMESPACE impl
} // END NAMESPACE threads
} // END NAMESPACE tinympi

#endif //_THREADS_ENV_H_
