#ifndef __MALLOC_OVERWRITE_H_
#define __MALLOC_OVERWRITE_H_

#include <dlfcn.h>
#include <stdio.h>
#include <stdexcept>
using hook_malloc_t = void(*)(void*, size_t);
using real_malloc_t = void*(*)(size_t);
using real_memalign_t = void*(*)(size_t, size_t);

extern real_malloc_t real_malloc;
extern real_memalign_t real_memalign;

inline void set_malloc_hook(hook_malloc_t hook_function) {
	real_malloc = (real_malloc_t) dlsym(RTLD_DEFAULT, "real_malloc_");
	real_memalign = (real_memalign_t) dlsym(RTLD_DEFAULT, "real_memalign_");

	using func_t = void(*)(hook_malloc_t);
	func_t func = (func_t) dlsym(RTLD_DEFAULT, "set_malloc_hook_");
	if (func == nullptr) {
		fprintf(stderr, "Error in dlsym: %s. Did you preload malloc_overwrite?\n", dlerror());
		return;
	}
	func(hook_function);
}
inline void* malloc_(size_t sz) {
	if (real_malloc == nullptr) {
		real_malloc = (real_malloc_t) dlsym(RTLD_DEFAULT, "real_malloc_");
		if (real_malloc == nullptr) {
			// here we are allowed to print, because malloc_ does not exist, so we will fail anyway
			fprintf(stderr, "Error in dlsym: %s\n", dlerror());
			return nullptr;
		}
	}
	return real_malloc(sz);
}
inline void* memalign_(size_t alignment, size_t size) {
	if (real_memalign == nullptr) {
		real_memalign = (real_memalign_t) dlsym(RTLD_DEFAULT, "real_memalign_");
		if (real_memalign == nullptr) {
			// here we are allowed to print, because malloc_ does not exist, so we will fail anyway
			fprintf(stderr, "Error in dlsym: %s\n", dlerror());
			return nullptr;
		}
	}
	return real_memalign(alignment, size);
}
#endif
